@@ -17,6 +17,7 @@
 package minio
 
 import (
+	"net/http"
 	"reflect"
 	"strings"
 	"testing"
@@ -58,17 +59,17 @@ func TestCalculateEvenSplits(t *testing.T) {
 	testCases := []struct {
 		// input size and source object
 		size int64
-		src  SourceInfo
+		src  CopySrcOptions
 
 		// output part-indexes
 		starts, ends []int64
 	}{
-		{0, SourceInfo{start: -1}, nil, nil},
-		{1, SourceInfo{start: -1}, []int64{0}, []int64{0}},
-		{1, SourceInfo{start: 0}, []int64{0}, []int64{0}},
+		{0, CopySrcOptions{Start: -1}, nil, nil},
+		{1, CopySrcOptions{Start: -1}, []int64{0}, []int64{0}},
+		{1, CopySrcOptions{Start: 0}, []int64{0}, []int64{0}},
 
-		{gb1, SourceInfo{start: -1}, []int64{0, 536870912}, []int64{536870911, 1073741823}},
-		{gb5, SourceInfo{start: -1},
+		{gb1, CopySrcOptions{Start: -1}, []int64{0, 536870912}, []int64{536870911, 1073741823}},
+		{gb5, CopySrcOptions{Start: -1},
 			[]int64{0, 536870912, 1073741824, 1610612736, 2147483648, 2684354560,
 				3221225472, 3758096384, 4294967296, 4831838208},
 			[]int64{536870911, 1073741823, 1610612735, 2147483647, 2684354559, 3221225471,
@@ -76,13 +77,13 @@ func TestCalculateEvenSplits(t *testing.T) {
 		},
 
 		// 2 part splits
-		{gb5p1, SourceInfo{start: -1},
+		{gb5p1, CopySrcOptions{Start: -1},
 			[]int64{0, 536870913, 1073741825, 1610612737, 2147483649, 2684354561,
 				3221225473, 3758096385, 4294967297, 4831838209},
 			[]int64{536870912, 1073741824, 1610612736, 2147483648, 2684354560, 3221225472,
 				3758096384, 4294967296, 4831838208, 5368709120},
 		},
-		{gb5p1, SourceInfo{start: -1},
+		{gb5p1, CopySrcOptions{Start: -1},
 			[]int64{0, 536870913, 1073741825, 1610612737, 2147483649, 2684354561,
 				3221225473, 3758096385, 4294967297, 4831838209},
 			[]int64{536870912, 1073741824, 1610612736, 2147483648, 2684354560, 3221225472,
@@ -90,7 +91,7 @@ func TestCalculateEvenSplits(t *testing.T) {
 		},
 
 		// 3 part splits
-		{gb10p1, SourceInfo{start: -1},
+		{gb10p1, CopySrcOptions{Start: -1},
 			[]int64{0, 536870913, 1073741825, 1610612737, 2147483649, 2684354561,
 				3221225473, 3758096385, 4294967297, 4831838209, 5368709121,
 				5905580033, 6442450945, 6979321857, 7516192769, 8053063681,
@@ -100,7 +101,7 @@ func TestCalculateEvenSplits(t *testing.T) {
 				5905580032, 6442450944, 6979321856, 7516192768, 8053063680,
 				8589934592, 9126805504, 9663676416, 10200547328, 10737418240},
 		},
-		{gb10p2, SourceInfo{start: -1},
+		{gb10p2, CopySrcOptions{Start: -1},
 			[]int64{0, 536870913, 1073741826, 1610612738, 2147483650, 2684354562,
 				3221225474, 3758096386, 4294967298, 4831838210, 5368709122,
 				5905580034, 6442450946, 6979321858, 7516192770, 8053063682,
@@ -120,8 +121,7 @@ func TestCalculateEvenSplits(t *testing.T) {
 	}
 }
 
-func TestGetUserMetaHeadersMap(t *testing.T) {
-
+func TestDestOptions(t *testing.T) {
 	userMetadata := map[string]string{
 		"test":                "test",
 		"x-amz-acl":           "public-read-write",
@@ -130,26 +130,27 @@ func TestGetUserMetaHeadersMap(t *testing.T) {
 		"x-amz-grant-write":   "test@exo.ch",
 	}
 
-	destInfo := &DestinationInfo{"bucket", "object", nil, userMetadata}
+	r := make(http.Header)
 
-	r := destInfo.getUserMetaHeadersMap(true)
+	dst := CopyDestOptions{
+		Bucket:          "bucket",
+		Object:          "object",
+		ReplaceMetadata: true,
+		UserMetadata:    userMetadata,
+	}
+	dst.Marshal(r)
 
-	i := 0
-
-	if _, ok := r["x-amz-metadata-directive"]; !ok {
-		t.Errorf("Test %d - metadata directive was expected but is missing", i)
-		i++
+	if v := r.Get("x-amz-metadata-directive"); v != "REPLACE" {
+		t.Errorf("Test - metadata directive was expected but is missing")
 	}
 
 	for k := range r {
 		if strings.HasSuffix(k, "test") && !strings.HasPrefix(k, "x-amz-meta-") {
-			t.Errorf("Test %d - meta %q was expected as an x amz meta", i, k)
-			i++
+			t.Errorf("Test meta %q was expected as an x amz meta", k)
 		}
 
 		if !strings.HasSuffix(k, "test") && strings.HasPrefix(k, "x-amz-meta-") {
-			t.Errorf("Test %d - an amz/standard/storageClass Header was expected but got an x amz meta data", i)
-			i++
+			t.Errorf("Test an amz/standard/storageClass Header was expected but got an x amz meta data")
 		}
 	}
 }
