@@ -1,6 +1,6 @@
 /*
- * Minio Go Library for Amazon S3 Compatible Cloud Storage
- * Copyright 2015-2017 Minio, Inc.
+ * MinIO Go Library for Amazon S3 Compatible Cloud Storage
+ * Copyright 2015-2017 MinIO, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,12 +18,14 @@
 package policy
 
 import (
-	"encoding/json"
 	"testing"
 
-	"github.com/minio/minio-go/pkg/set"
+	jsoniter "github.com/json-iterator/go"
+	"github.com/minio/minio-go/v7/pkg/set"
 )
 
+var json = jsoniter.ConfigCompatibleWithStandardLibrary
+
 // ConditionKeyMap.Add() is called and the result is validated.
 func TestConditionKeyMapAdd(t *testing.T) {
 	condKeyMap := make(ConditionKeyMap)
