@@ -1,6 +1,6 @@
 /*
- * Minio Go Library for Amazon S3 Compatible Cloud Storage
- * Copyright 2015-2017 Minio, Inc.
+ * MinIO Go Library for Amazon S3 Compatible Cloud Storage
+ * Copyright 2015-2017 MinIO, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -19,12 +19,11 @@ package minio
 
 import (
 	"fmt"
-	"net/http"
 	"net/url"
 	"testing"
 	"time"
 
-	"github.com/minio/minio-go/pkg/s3utils"
+	"github.com/minio/minio-go/v7/pkg/s3utils"
 )
 
 // Tests signature redacting function used
@@ -52,21 +51,6 @@ func TestRedactSignature(t *testing.T) {
 	}
 }
 
-// Tests filter header function by filtering out
-// some custom header keys.
-func TestFilterHeader(t *testing.T) {
-	header := http.Header{}
-	header.Set("Content-Type", "binary/octet-stream")
-	header.Set("Content-Encoding", "gzip")
-	newHeader := filterHeader(header, []string{"Content-Type"})
-	if len(newHeader) > 1 {
-		t.Fatalf("Unexpected size of the returned header, should be 1, got %d", len(newHeader))
-	}
-	if newHeader.Get("Content-Encoding") != "gzip" {
-		t.Fatalf("Unexpected content-encoding value, expected 'gzip', got %s", newHeader.Get("Content-Encoding"))
-	}
-}
-
 // Tests for 'getEndpointURL(endpoint string, inSecure bool)'.
 func TestGetEndpointURL(t *testing.T) {
 	testCases := []struct {
@@ -82,16 +66,18 @@ func TestGetEndpointURL(t *testing.T) {
 	}{
 		{"s3.amazonaws.com", true, "https://s3.amazonaws.com", nil, true},
 		{"s3.cn-north-1.amazonaws.com.cn", true, "https://s3.cn-north-1.amazonaws.com.cn", nil, true},
+		{"s3.cn-northwest-1.amazonaws.com.cn", true, "https://s3.cn-northwest-1.amazonaws.com.cn", nil, true},
 		{"s3.amazonaws.com", false, "http://s3.amazonaws.com", nil, true},
 		{"s3.cn-north-1.amazonaws.com.cn", false, "http://s3.cn-north-1.amazonaws.com.cn", nil, true},
+		{"s3.cn-northwest-1.amazonaws.com.cn", false, "http://s3.cn-northwest-1.amazonaws.com.cn", nil, true},
 		{"192.168.1.1:9000", false, "http://192.168.1.1:9000", nil, true},
 		{"192.168.1.1:9000", true, "https://192.168.1.1:9000", nil, true},
 		{"s3.amazonaws.com:443", true, "https://s3.amazonaws.com:443", nil, true},
-		{"13333.123123.-", true, "", ErrInvalidArgument(fmt.Sprintf("Endpoint: %s does not follow ip address or domain name standards.", "13333.123123.-")), false},
-		{"13333.123123.-", true, "", ErrInvalidArgument(fmt.Sprintf("Endpoint: %s does not follow ip address or domain name standards.", "13333.123123.-")), false},
-		{"storage.googleapis.com:4000", true, "", ErrInvalidArgument("Google Cloud Storage endpoint should be 'storage.googleapis.com'."), false},
-		{"s3.aamzza.-", true, "", ErrInvalidArgument(fmt.Sprintf("Endpoint: %s does not follow ip address or domain name standards.", "s3.aamzza.-")), false},
-		{"", true, "", ErrInvalidArgument("Endpoint:  does not follow ip address or domain name standards."), false},
+		{"13333.123123.-", true, "", errInvalidArgument(fmt.Sprintf("Endpoint: %s does not follow ip address or domain name standards.", "13333.123123.-")), false},
+		{"13333.123123.-", true, "", errInvalidArgument(fmt.Sprintf("Endpoint: %s does not follow ip address or domain name standards.", "13333.123123.-")), false},
+		{"storage.googleapis.com:4000", true, "", errInvalidArgument("Google Cloud Storage endpoint should be 'storage.googleapis.com'."), false},
+		{"s3.aamzza.-", true, "", errInvalidArgument(fmt.Sprintf("Endpoint: %s does not follow ip address or domain name standards.", "s3.aamzza.-")), false},
+		{"", true, "", errInvalidArgument("Endpoint:  does not follow ip address or domain name standards."), false},
 	}
 
 	for i, testCase := range testCases {
@@ -126,7 +112,7 @@ func TestIsValidEndpointURL(t *testing.T) {
 		// Flag indicating whether the test is expected to pass or not.
 		shouldPass bool
 	}{
-		{"", ErrInvalidArgument("Endpoint url cannot be empty."), false},
+		{"", errInvalidArgument("Endpoint url cannot be empty."), false},
 		{"/", nil, true},
 		{"https://s3.amazonaws.com", nil, true},
 		{"https://s3.cn-north-1.amazonaws.com.cn", nil, true},
@@ -136,10 +122,10 @@ func TestIsValidEndpointURL(t *testing.T) {
 		{"https://storage.googleapis.com/", nil, true},
 		{"https://z3.amazonaws.com", nil, true},
 		{"https://mybalancer.us-east-1.elb.amazonaws.com", nil, true},
-		{"192.168.1.1", ErrInvalidArgument("Endpoint url cannot have fully qualified paths."), false},
-		{"https://amazon.googleapis.com/", ErrInvalidArgument("Google Cloud Storage endpoint should be 'storage.googleapis.com'."), false},
-		{"https://storage.googleapis.com/bucket/", ErrInvalidArgument("Endpoint url cannot have fully qualified paths."), false},
-		{"https://s3.amazonaws.com/bucket/object", ErrInvalidArgument("Endpoint url cannot have fully qualified paths."), false},
+		{"192.168.1.1", errInvalidArgument("Endpoint url cannot have fully qualified paths."), false},
+		{"https://amazon.googleapis.com/", errInvalidArgument("Google Cloud Storage endpoint should be 'storage.googleapis.com'."), false},
+		{"https://storage.googleapis.com/bucket/", errInvalidArgument("Endpoint url cannot have fully qualified paths."), false},
+		{"https://s3.amazonaws.com/bucket/object", errInvalidArgument("Endpoint url cannot have fully qualified paths."), false},
 	}
 
 	for i, testCase := range testCases {
@@ -200,7 +186,13 @@ func TestDefaultBucketLocation(t *testing.T) {
 			regionOverride:   "",
 			expectedLocation: "cn-north-1",
 		},
-		// No region provided, no standard region strings provided as well. - Test 5.
+		// China region should be honored, region override not provided. - Test 5.
+		{
+			endpointURL:      url.URL{Host: "s3.cn-northwest-1.amazonaws.com.cn"},
+			regionOverride:   "",
+			expectedLocation: "cn-northwest-1",
+		},
+		// No region provided, no standard region strings provided as well. - Test 6.
 		{
 			endpointURL:      url.URL{Host: "s3.amazonaws.com"},
 			regionOverride:   "",
@@ -226,9 +218,9 @@ func TestIsValidExpiry(t *testing.T) {
 		// Flag to indicate whether the test should pass.
 		shouldPass bool
 	}{
-		{100 * time.Millisecond, ErrInvalidArgument("Expires cannot be lesser than 1 second."), false},
-		{604801 * time.Second, ErrInvalidArgument("Expires cannot be greater than 7 days."), false},
-		{0 * time.Second, ErrInvalidArgument("Expires cannot be lesser than 1 second."), false},
+		{100 * time.Millisecond, errInvalidArgument("Expires cannot be lesser than 1 second."), false},
+		{604801 * time.Second, errInvalidArgument("Expires cannot be greater than 7 days."), false},
+		{0 * time.Second, errInvalidArgument("Expires cannot be lesser than 1 second."), false},
 		{1 * time.Second, nil, true},
 		{10000 * time.Second, nil, true},
 		{999 * time.Second, nil, true},
@@ -262,12 +254,12 @@ func TestIsValidBucketName(t *testing.T) {
 		// Flag to indicate whether test should Pass.
 		shouldPass bool
 	}{
-		{".mybucket", ErrInvalidBucketName("Bucket name contains invalid characters"), false},
-		{"mybucket.", ErrInvalidBucketName("Bucket name contains invalid characters"), false},
-		{"mybucket-", ErrInvalidBucketName("Bucket name contains invalid characters"), false},
-		{"my", ErrInvalidBucketName("Bucket name cannot be smaller than 3 characters"), false},
-		{"", ErrInvalidBucketName("Bucket name cannot be empty"), false},
-		{"my..bucket", ErrInvalidBucketName("Bucket name contains invalid characters"), false},
+		{".mybucket", errInvalidBucketName("Bucket name contains invalid characters"), false},
+		{"mybucket.", errInvalidBucketName("Bucket name contains invalid characters"), false},
+		{"mybucket-", errInvalidBucketName("Bucket name contains invalid characters"), false},
+		{"my", errInvalidBucketName("Bucket name cannot be shorter than 3 characters"), false},
+		{"", errInvalidBucketName("Bucket name cannot be empty"), false},
+		{"my..bucket", errInvalidBucketName("Bucket name contains invalid characters"), false},
 		{"my.bucket.com", nil, true},
 		{"my-bucket", nil, true},
 		{"123my-bucket", nil, true},
@@ -304,6 +296,7 @@ func TestIsStandardHeader(t *testing.T) {
 		{"content-type", true},
 		{"cache-control", true},
 		{"content-disposition", true},
+		{"content-language", true},
 		{"random-header", false},
 	}
 
@@ -341,32 +334,6 @@ func TestIsSSEHeader(t *testing.T) {
 	}
 }
 
-// Tests if header is client encryption header
-func TestIsCSEHeader(t *testing.T) {
-	testCases := []struct {
-		// Input.
-		header string
-		// Expected result.
-		expectedValue bool
-	}{
-		{"x-amz-iv", true},
-		{"x-amz-key", true},
-		{"x-amz-matdesc", true},
-		{"x-amz-meta-x-amz-iv", true},
-		{"x-amz-meta-x-amz-key", true},
-		{"x-amz-meta-x-amz-matdesc", true},
-		{"random-header", false},
-	}
-
-	for i, testCase := range testCases {
-		actual := isCSEHeader(testCase.header)
-		if actual != testCase.expectedValue {
-			t.Errorf("Test %d: Expected to pass, but failed", i+1)
-		}
-	}
-
-}
-
 // Tests if header is x-amz-meta or x-amz-acl
 func TestIsAmzHeader(t *testing.T) {
 	testCases := []struct {
